"""Implements utility functions and classes used by nose.core
"""
import os
import re
import sys

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

    
def absdir(path):
    """Return absolute, normalized path to directory, if it exists; None
    otherwise.
    """
    if not path.startswith(os.path.sep):
        path = os.path.normpath(os.path.abspath(os.path.join(os.getcwd(),
                                                             path)))
    if path is None or not os.path.isdir(path):
        return None
    return path


def absfile(path, where=None):
    """Return absolute, normalized path to file (optionally in directory
    where), or None if the file can't be found either in where or the current
    working directory.
    """
    orig = path
    if where is None:
        where = os.getcwd()
    if path[0] != os.path.sep:
        path = os.path.normpath(os.path.abspath(os.path.join(where, path)))
    if path is None or not os.path.exists(path):
        if where != os.getcwd():
            # try the cwd instead
            path = os.path.normpath(os.path.abspath(os.path.join(os.getcwd(),
                                                                 orig)))
    if path is None or not os.path.exists(path):
        return None
    if os.path.isdir(path):
        # might want in __init__.py from pacakge
        init = os.path.join(path,'__init__.py')
        if os.path.isfile(init):
            return init
    elif os.path.isfile(path):
        return path
    return None

    
def msg(message, min_verbosity=0, stream=None):
    """Output a status message, if configured verbosity is greater than
    the minimum verbosity rating of the message.
    """
    if Config.verbosity > min_verbosity:
        if stream is None:
            stream = sys.stderr
        stream.write("  " * (min_verbosity-1))
        stream.write(message)
        stream.write("\n")

        
class _Capture(object):
    """Output capture handler
    """
    def __init__(self):
        self.buffer = None
        self.orig_stdout = sys.stdout

    def begin(self):
        """Start capturing output
        """
        self.buffer = StringIO()
        sys.stdout = self.buffer
        msg("begin capture", 6)

    def captured(self):
        """Return buffer of captured output
        """
        if self.buffer is not None:
            return self.buffer.getvalue()
        
    def __call__(self):
        self.begin()

    def end(self):
        """In the end as in the beginning...
        """
        msg("end capture", 6)
        self.buffer = None
        sys.stdout = self.orig_stdout
        
Capture = _Capture()


class _Config(object):
    """nose configuration. For internal use only.
    """
    testMatch = re.compile(r'(?:^|[\b_\.-])[Tt]est')
    capture = True
    coverage = False
    cover_tests = False
    cover_packages = None
    detailed_errors = False
    exclude = None
    file_only = None
    ignore_private = True
    include = None
    module_only = None
    precover_modules = []

    verbosity = 1
    where = None

    def __str__(self):
        return str({
                'testMatch': self.testMatch,
                'detailed_errors': self.detailed_errors,
                'include': self.include,
                'exclude': self.exclude,
                'capture': self.capture,
                'verbosity': self.verbosity,
                'where': self.where })

    def wantModuleCoverage(self, name, module):
        if name in self.precover_modules:
            return False
        if not hasattr(module, '__file__'):
            return False
        root, ext = os.path.splitext(module.__file__)
        if not ext in ('.py', '.pyc', '.pyo'):
            return False
        if self.cover_packages is not None:
            for package in self.cover_packages:
                if (name.startswith(package)
                    and (self.cover_tests
                         or not self.testMatch.search(name))):
                    return True
        if self.testMatch.search(name) and not self.cover_tests:
            return False
        # accept any package that passed the previous tests, unless
        # cover_packages is on -- in that case, if we wanted this
        # module, we would have already returned True
        return (self.cover_packages is None)
    
    def wantTestsFromFile(self, file):
        if self.file_only is None:
            return True
        else:
            # file extension on one or the other might be .py or py[c|o]
            want_root, want_ext = os.path.splitext(self.file_only)
            candidate_root, candidate_ext = os.path.splitext(file)
            if candidate_root == want_root:
                if (candidate_ext == want_ext or
                    (candidate_ext.startswith('.py') and
                     want_ext.startswith('.py'))):
                    return True
            msg("Skipping tests in '%s' (want '%s')" %
                (file, self.file_only), 3)
            return False
        
    def wantTest(self, name, test_case=False, file=None, package=None):
        """Detemine whether a given name is a wanted test
        """
        # testcase subclasses get a pass on matching testMatch
        msg("is %s (%s) (%s) (%s) a wanted test?" %
            (name, test_case, file, package), 5)
        if file is not None:
            if not self.wantTestsFromFile(file):
                return False
            if package:
                # examine all python files in packages, except __init__
                # (__init__ elements are in the package globals)
                return (name != '__init__')
        if not test_case and not self.testMatch.search(name):
            msg("%s does not match testMatch" % name, 5)
            return False
        if self.include and not self.include.search(name):
            msg("%s does not match include" % name, 5)
            return False
        if self.exclude and self.exclude.search(name):
            msg("%s matches exclude" % name, 5)
            return False
        if (self.module_only
            and package is not None
            and not package.startswith(self.module_only)):
            msg("%s.%s is not in module %s" % 
                (package, name, self.module_only), 5)
            return False
        if self.ignore_private and name.startswith('_'):
            return False
        msg("%s is a test" % name, 5)
        return True    
Config = _Config()


def start_coverage():
    """Start measuring code coverage"""
    import atexit
    import coverage
    Config.precover_modules = sys.modules.keys()[:]
    coverage.start()
    atexit.register(coverage_report)

    
def coverage_report():
    """End code coverage and output report"""
    import coverage
    import types
    coverage.stop()

    modules = [ module
                for name, module in sys.modules.items()
                if Config.wantModuleCoverage(name, module) ]
    coverage.report(modules, file=sys.stderr)
    
